# -*- coding: utf-8 -*-

from odoo import http
from odoo.http import request
from odoo import tools
from odoo.tools.translate import _
from collections import OrderedDict
from odoo.fields import Date
import re
from datetime import date


class website_portal(http.Controller):

    """
    Mandatory fields in the portal details of the user
    """
    MANDATORY_DETAILS_FIELDS = ["name", "phone", "email",
                                "street", "city", "country_id",
                                "acc_number", "bank_name",
                                "paypal_email"]
    """
    Optional fields in the portal details of the user
    """
    OPTIONAL_DETAILS_FIELDS = [
        "mobile", "zipcode", "state_id", "vat",
        "company_name", "bic", "beneficiary", "acc_country_id",
        "aba", "fed_wire", "comment_invoice", "street2"]

    REQUEST_FIELDS = ["stage", "justify"]

    _items_per_page = 20

    """
    Prepare the values to render portal layout
    """
    def _prepare_portal_layout_values(self):
        partner = request.env.user.partner_id
        count_project = request.env['project.project'].sudo().search_count([
            ('privacy_visibility', '=', 'portal')])
        count_task = request.env['project.task'].sudo().search_count([
            ('project_id.privacy_visibility', '=', 'portal')])
        if partner.user_id:
            sales_rep = partner.user_id
        else:
            sales_rep = False
        values = {
            'sales_rep': sales_rep,
            'company': request.website.company_id,
            'user': request.env.user,
            'count_project': count_project,
            'count_task': count_task
        }
        return values

    """

    """
    def _get_archive_groups(self, model, domain=None, fields=None,
                            groupby="create_date", order="create_date desc"):
        if not model:
            return []
        if domain is None:
            domain = []
        if fields is None:
            fields = ['name', 'create_date']
        groups = []
        for group in request.env[model]._read_group_raw(
                domain, fields=fields, groupby=groupby, orderby=order):
            dates, label = group[groupby]
            date_begin, date_end = dates.split('/')
            groups.append({
                'date_begin': Date.to_string(Date.from_string(date_begin)),
                'date_end': Date.to_string(Date.from_string(date_end)),
                'name': label,
                'item_count': group[groupby + '_count']
            })
        return groups

    """
    Function to notify the administrator if the portal user has
    accepted or rejected the task.
    """
    def send_mail_with_wizard(self, stage, text, task):
        if text:
            text = text.split('\n')
        if str(stage) == 'accept':
            body = (_("<p>Task accepted.</p>"))
        elif str(stage) == 'refuse':
            if text:
                body = (_("<p>Task rejected:<br/>"))
                for t in text:
                    if t != text[len(text)-1]:
                        body += ("%s<br/>" % t)
                    else:
                        body += ("%s</p>" % t)
            else:
                body = (_("<p>Task rejected without justification.</p>"))
        elif str(stage) == 'deliver':
            body = (_("<p>The task has been delivered.</p>"))
        if body:
            request.env['mail.message'].sudo().create({
                'model': 'project.task',
                'author_id': task.user_id.partner_id.id,
                'res_id': task.id,
                'body': body,
                'message_type': 'comment'
            })

    """
    Controller that generates the portal user's tasks data to be
    loaded into the view.
    """
    @http.route(['/my/list_task', '/my/list_task/page/<int:page>'],
                type='http', auth="user", website=True)
    def my_tasks(self, page=1, date_begin=None, date_end=None,
                 project=None, sortby=None, **kw):
        values = self._prepare_portal_layout_values()

        sortings = {
            'date': {'label': _('Newest'), 'order': 'create_date desc'},
            'name': {'label': _('Name'), 'order': 'name'},
            'stage': {'label': _('Stage'), 'order': 'stage_id'},
            'update': {
             'label': _(
                'Last Stage Update'), 'order': 'date_last_stage_update desc'},
        }

        domain = [
            ('project_id.privacy_visibility', '=', 'portal'),
            ('stage_id', '!=', 'Pending'),
            ('stage_id', '!=', 'Assigned'),
            ('stage_id', '!=', 'Validated'),
            ('stage_id', '!=', 'To Invoice'),
            ('stage_id', '!=', 'Invoiced'),
            ('stage_id', '!=', 'On Hold'),
            ('stage_id', '!=', 'Done'),
            ('stage_id', '!=', 'Cancelled'),
            ('stage_id', '!=', False),
        ]

        if values['user'].id in request.env['res.groups'].sudo().search([
                ('name', '=', 'Portal'), ('is_portal', '=', True)]).users.ids:
            domain += [('user_id', '=', values['user'].id)]

        # archive groups - Default Group By 'create_date'
        archive_groups = self._get_archive_groups('project.task', domain)
        if date_begin and date_end:
            domain += [
                ('create_date', '>', date_begin),
                ('create_date', '<=', date_end)
            ]

        tasks = request.env['project.task'].sudo().search(domain)
        if tasks:
            porj_ids = []
            for task in tasks:
                porj_ids.append(task.project_id.id)
            projects = request.env['project.project'].sudo().search([
                ('id', 'in', porj_ids)])
        else:
            projects = request.env['project.project']

        project_filters = {
            'all': {'label': _('All'), 'domain': []},
        }

        for task in tasks:
            project_filters.update({
                str(task.project_id.id): {
                    'label': task.project_id.name,
                    'domain': [('project_id', '=', task.project_id.id)]
                }
            })

        domain += project_filters.get(
            project, project_filters['all'])['domain']
        order = sortings.get(sortby, sortings['date'])['order']

        # pager
        count = request.env['project.task'].search_count(domain)
        pager = request.website.pager(
            url="/my/list_task",
            url_args={
                'date_begin': date_begin,
                'date_end': date_end,
                'sortby': sortby,
                'project': project
            },
            total=count,
            page=page,
            step=self._items_per_page
        )

        # content according to pager and archive selected
        tasks = request.env['project.task'].sudo().search(
            domain, order=order, limit=self._items_per_page,
            offset=(page - 1) * self._items_per_page
        )

        values.update({
            'date': date_begin,
            'date_end': date_end,
            'project_filters': OrderedDict(sorted(project_filters.items())),
            'projects': projects,
            'project': project,
            'sortings': sortings,
            'sortby': sortby,
            'tasks': tasks,
            'page_name': 'task',
            'archive_groups': archive_groups,
            'defauurl': '/my/list_task',
            'pager': pager
        })
        return request.render("website_project.my_tasks", values)

    """
    Controller that generates the specified portal user's task data to be
    loaded into the view.
    """
    @http.route(['/my/task/<int:task>'], type='http',
                auth="user", website=True)
    def my_task(self, task=None, **kw):
        user = request.env.user
        is_portal = user.id in request.env['res.groups'].sudo().search([
            ('name', '=', 'Portal'), ('is_portal', '=', True)]).users.ids
        if task:
            task_id = request.env['project.task'].sudo().search([
                ('id', '=', task), ('user_id', '=', user.id)])
            timesheets_ids = request.env[
                'account.analytic.line'].sudo().search([
                    ('task_id', '=', task),
                    ('user_id', '=', user.id)
                ])
        if kw and 'stage' in kw:
            if task_id.stage_id.name == 'Sent':
                if kw['stage'] == 'accept':
                    task_id.stage_id = request.env['project.task.type'].search(
                        [('name', '=', 'Accepted')])
                    self.send_mail_with_wizard(
                        kw['stage'], kw.get('comment', False), task_id)
                elif kw['stage'] == 'refuse':
                    if kw.get('comment', False):
                        task_id.stage_id = request.env[
                            'project.task.type'].search([
                                ('name', '=', 'Refused')])
                        self.send_mail_with_wizard(
                            kw['stage'], kw.get('comment', False), task_id)
            if task_id.stage_id.name in ('In Progress', 'Accepted'):
                if kw['stage'] == 'deliver':
                    task_id.stage_id = request.env['project.task.type'].search(
                        [('name', '=', 'Delivered')])
                    self.send_mail_with_wizard(
                        kw['stage'], False, task_id)
        values = {
            'task': task_id,
            'user': user,
            'timesheets': timesheets_ids,
            'is_portal': is_portal,
        }
        return request.render(
            "website_project.my_task",
            values
        )

    """
    Controller that generates the data of the tasks that are ready to be
    invoiced by the user portal.
    """
    @http.route(
            ['/my/list_invoice_tasks',
             '/my/list_invoice_tasks/<int:task>',
             '/my/list_invoice_tasks/page/<int:page>'],
            type='http', auth="user", website=True)
    def my_invoice_tasks(self, page=1, task=None, sortby=None, **kw):
        if task:
            task_id = request.env['project.task'].sudo().browse(task)
            if task_id.stage_id.name == 'Validated':
                task_id.stage_id = request.env['project.task.type'].search([
                    ('name', '=', 'To Invoice')
                ])
            elif task_id.stage_id.name == 'To Invoice':
                task_id.stage_id = request.env['project.task.type'].search([
                    ('name', '=', 'Validated')
                ])
            return request.redirect('/my/list_invoice_tasks')

        user = request.env.user
        ''' También queremos que se puedan facturar tareas archivadas '''
        invoice_task_ids = request.env[
           'project.task'].with_context(active_test=False).sudo().search([
            ('stage_id.name', 'in', ['Validated', 'To Invoice']),
            ('user_id', '=', user.id)]
        )
        is_portal = user.id in request.env['res.groups'].sudo().search([
            ('name', '=', 'Portal'), ('is_portal', '=', True)]).users.ids

        error_info_paypal = False
        error_info_bank = False
        input_number = False
        invoice = False
        if kw:
            if kw.get('invoice', False):
                if user.partner_id.parent_id:
                    parent = user.partner_id.parent_id
                    if parent.paypal_email:
                        error_info_paypal = not parent.paypal_email
                    else:
                        error_info_bank = not parent.bank_ids
                else:
                    partner = user.partner_id
                    if partner.paypal_email:
                        error_info_paypal = not partner.paypal_email
                    else:
                        error_info_bank = not partner.bank_ids

                if not error_info_paypal and not error_info_bank:
                    input_number = True
            if kw.get('invoice_number', False):
                invoice = self._create_draft_invoice(
                    invoice_task_ids, user.partner_id,
                    kw['invoice_number'], kw['invoice_comment']
                )

        total = 0
        to_invoice_tasks = False
        for task in invoice_task_ids:
            if task.stage_id.name == 'To Invoice':
                total += (task.price_hour * task.dedicated_hours)
                to_invoice_tasks = True

        values = {
            'user': user,
            'is_portal': is_portal,
            'invoice_tasks': invoice_task_ids,
            'total': total,
            'error_info_bank': error_info_bank,
            'error_info_paypal': error_info_paypal,
            'input_number': input_number,
            'invoice': invoice,
            'history': False,
            'invoice_ref': False,
            'page_name': 'tasks',
            'comment': user.partner_id.comment_invoice_portal,
            'to_invoice_tasks': to_invoice_tasks,

        }
        return request.render("extern_user_website.my_list_invoice_tasks", values)

    """
    Function that generates a supplier invoice for the portal user
    when invoicing tasks.
    """
    def _create_draft_invoice(self, tasks, partner, number, comment):

        journal = request.env['account.journal'].sudo().search([
            ('code', '=', 'FACTU')
        ])

        invoice = request.env['account.invoice'].sudo().create({
            'type': 'in_invoice',
            'journal_id': journal.id,
            'partner_id': partner.id,
            'reference': number,
            'date_invoice': date.today(),
            'date': date.today(),
            'comment': comment,
        })

        for task in tasks.filtered(lambda x: x.stage_id.name == 'To Invoice'):
            acc = task.product_id.categ_id.property_account_expense_categ_id
            ''' También queremos que al facturar encuentre las cuentas
            analíticas archivadas. '''
            acc_analytic = request.env['account.analytic.account']\
                .with_context(active_test=False).sudo().search([
                    ('name', '=', task.project_id.name)
                ])
            if acc_analytic:
                acc_analytic = acc_analytic.id
            else:
                acc_analytic = False
            line = request.env['account.invoice.line'].sudo().create({
                'product_id': task.product_id.id,
                'name': task.product_id.name,
                'price_unit': 0.0,
                'account_id': acc.id,
                'invoice_id': invoice.id,
                'task_id': task.id,
                'account_analytic_id': acc_analytic,
            })
            line._onchange_product_id()
            line.price_unit = (task.price_hour * task.dedicated_hours)
            line._set_taxes()

            task.stage_id = request.env['project.task.type'].search([
                ('name', '=', 'Invoiced')
            ])
        invoice.compute_taxes()

        return invoice

    """
    Controller that generates the details of a task performed by the
    portal user.
    """
    @http.route(
            ['/my/details_task/<int:task>',
                '/my/details_task/<int:task>/<int:modify>'],
            type='http', auth="user", website=True)
    def details_task(self, task=None, modify=None, sortby=None, **kw):
        if task:
            if modify:
                task_id = request.env['project.task'].sudo().browse(task)
                if task_id.stage_id.name == 'Validated':
                    task_id.stage_id = request.env[
                        'project.task.type'].search([
                            ('name', '=', 'To Invoice')
                        ])
                elif task_id.stage_id.name == 'To Invoice':
                    task_id.stage_id = request.env[
                        'project.task.type'].search([
                            ('name', '=', 'Validated')
                        ])
                return request.redirect('/my/details_task/%s' % task_id.id)
            user = request.env.user
            is_portal = user.id in request.env['res.groups'].sudo().search([
                ('name', '=', 'Portal'), ('is_portal', '=', True)]).users.ids
            task_id = request.env['project.task'].sudo().browse(task)
            invoiced = False
            line = request.env['account.invoice.line'].sudo().search([
                ('task_id', '=', task_id.id)], limit=1)
            if line:
                invoiced = line.invoice_id.id
            page_name = 'tasks'
            if invoiced:
                page_name = 'invoices'
            values = {
                'id': task_id.id,
                'user': user,
                'is_portal': is_portal,
                'lang': task_id.language_dest,
                'po': task_id.project_id.budget_code,
                'project': task_id.project_id.name,
                'price_hour': task_id.price_hour,
                'dedicated_hours': task_id.dedicated_hours,
                'amount': task_id.price_hour * task_id.dedicated_hours,
                'to_invoice': task_id.stage_id.name,
                'invoiced': invoiced,
                'page_name': page_name,
            }
            return request.render("extern_user_website.my_task_details", values)
        else:
            return request.redirect('/my/list_invoice_tasks')

    """
    Controller that loads the information to print supplier invoices
    for the portal user.
    """
    @http.route(
            ['/my/invoice_task',
             '/my/invoice_tasks/<int:invoice>'],
            type='http', auth="user", website=True)
    def my_print_invoice_task(self, invoice=None, sortby=None, **kw):
        if kw and kw.get('invoice_id', False):
            pdf = request.env['report'].sudo().get_pdf(
                [int(kw['invoice_id'])], 'extern_user_website.report_invoice_portal',
                data=None)
            pdfhttpheaders = [
                ('Content-Type', 'application/pdf'),
                ('Content-Length', len(pdf)),
                ('Content-Disposition', 'attachment; filename=Invoice.pdf;')
            ]
            return request.make_response(pdf, headers=pdfhttpheaders)
        elif invoice:
            pdf = request.env['report'].sudo().get_pdf(
                [invoice], 'extern_user_website.report_invoice_portal', data=None)
            pdfhttpheaders = [
                ('Content-Type', 'application/pdf'),
                ('Content-Length', len(pdf)),
                ('Content-Disposition', 'attachment; filename=Invoice.pdf;')
            ]
            return request.make_response(pdf, headers=pdfhttpheaders)
        else:
            return request.redirect('/my')

    """
    Controller that generates the information of the history of invoices
    of the portal user.
    """
    @http.route(
            ['/my/invoice_history',
             '/my/invoice_history/<int:invoice>',
             '/my/invoice_history/page/<int:page>'],
            type='http', auth="user", website=True)
    def my_invoice_history(self, page=1, invoice=None, sortby=None, **kw):
        user = request.env.user
        domain = [('partner_id', '=', user.partner_id.id),
                  ('date_invoice', '>', '2020-06-30')
                  ]

        if invoice:
            user = request.env.user
            invoice_id = request.env['account.invoice'].sudo().browse(invoice)
            invoice_task_ids = request.env['project.task']
            is_portal = user.id in request.env['res.groups'].sudo().search([
                ('name', '=', 'Portal'), ('is_portal', '=', True)]).users.ids

            for line in invoice_id.invoice_line_ids:
                if line.task_id:
                    invoice_task_ids += line.task_id

            error_info = False
            input_number = False
            invoice = False
            total = 0
            for task in invoice_task_ids:
                total += (task.sudo().price_hour * task.sudo().dedicated_hours)

            values = {
                'user': user,
                'is_portal': is_portal,
                'invoice_tasks': request.env[
                    'project.task'].sudo().browse(invoice_task_ids.ids),
                'total': total,
                'error_info': error_info,
                'input_number': input_number,
                'invoice': invoice,
                'history': True,
                'invoice_ref': invoice_id.reference,
                'page_name': 'invoices',
            }
            return request.render("extern_user_website.my_list_invoice_tasks", values)

        else:

            # pager
            count = request.env['account.invoice'].sudo().search_count(domain)
            pager = request.website.pager(
                url="/my/invoice_history",
                total=count,
                page=page,
                step=10
            )

            invoice_ids = request.env['account.invoice'].sudo().search(
                domain, limit=10, offset=(page - 1) * 10)

            values = {
                'user': user,
                'invoices': invoice_ids,
                'pager': pager,
                'page_name': 'invoices',
            }

            return request.render(
                "extern_user_website.my_invoice_history", values)

    """
    Controller that generates the information of the user's portal details.
    """
    @http.route(['/my/details'], type='http', auth='user', website=True)
    def portal_details(self, redirect=None, **post):
        partner = request.env.user.partner_id
        values = {
            'error': {},
            'error_message': []
        }
        if post:
            # AÑADIMOS EL NAME COMO FALSE SI EL PARTNER ES COMPAÑIA
            if partner.is_company:
                if 'name' not in post:
                    post['name'] = False

            # AÑADIMOS LOS CAMPOS RELACIONADOS COUNTRY Y STATE SI EL
            # PARTNER TIENE PADRE
            if partner.parent_id:
                if 'country_id' not in post and partner.parent_id.country_id:
                    post['country_id'] = partner.parent_id.country_id.id

                if 'state_id' not in post and partner.parent_id.state_id:
                    post['state_id'] = partner.parent_id.state_id.id

                if 'acc_country_id' not in post and\
                        partner.parent_id.bank_ids and\
                        partner.parent_id.bank_ids[0].acc_country_id:
                    post['acc_country_id'] =\
                        partner.parent_id.bank_ids[0].acc_country_id.code

            if 'acc_number' not in post:
                post['acc_number'] = False
            if 'bank_name' not in post:
                post['bank_name'] = False
            if 'paypal_email' not in post:
                post['paypal_email'] = False

            error, error_message =\
                self.portal_details_form_validate(post, partner)
            values.update({'error': error, 'error_message': error_message})
            values.update(post)
            if not error:
                values =\
                    {key: post[key]for key in self.MANDATORY_DETAILS_FIELDS}
                values.update(
                    {key: post[key]
                     for key in self.OPTIONAL_DETAILS_FIELDS if key in post})
                values.update({'zip': values.pop('zipcode', '')})
                # Confirm details
                self.confirm_details(partner, post)
                if redirect:
                    return request.redirect(redirect)

                return request.render("extern_user_website.portal_details_message")

        countries = request.env['res.country'].sudo().search([])
        states = request.env['res.country.state'].sudo().search([])

        values.update({
            'partner': partner,
            'countries': countries,
            'states': states,
            'has_check_vat': hasattr(request.env['res.partner'], 'check_vat'),
            'redirect': redirect,
        })
        return request.render("extern_user_website.portal_details", values)

    """
    Controller that generates the credential information of the portal user.
    """
    @http.route(['/my/credentials'], type='http', auth='user', website=True)
    def portal_credentials(self, redirect=None, **post):
        partner = request.env.user.partner_id
        values = {
            'partner': partner
        }
        return request.render("extern_user_website.portal_credentials", values)

    """
    Function that checks the information entered by the user portal
    in the form details.
    """
    def portal_details_form_validate(self, data, partner):
        error = dict()
        error_message = []

        # Validation
        for field_name in self.MANDATORY_DETAILS_FIELDS:
            if field_name == 'name':
                if not data.get('name', False) and not partner.is_company:
                    error[field_name] = 'missing'
                continue

            if field_name == 'phone':
                if not data.get('phone') and not data.get('mobile'):
                    error["phone"] = 'error'
                    error["mobile"] = 'error'
                    error_message.append(
                        _('At least the phone field or the mobile '
                          'field must be filled.'))
                continue

            if (field_name == 'street' or field_name == 'city' or
                    field_name == 'country_id'):
                if not partner.parent_id and not data.get(field_name):
                    error[field_name] = 'missing'
                continue

        # email validation
        if (data.get('email') and
                not tools.single_email_re.match(data.get('email'))):
            error["email"] = 'error'
            error_message.append(
                _('Invalid Email! Please enter a valid email address.'))

        if not partner.parent_id:
            # vat validation
            if (data.get("vat") and
                    hasattr(request.env["res.partner"], "check_vat")):
                if data.get("country_id"):
                    data["vat"] = request.env["res.partner"].fix_eu_vat_number(
                        int(data.get("country_id")), data.get("vat"))
                if request.website.company_id.sudo().vat_check_vies:
                    # force full VIES online check
                    check_func = request.env["res.partner"].vies_vat_check
                else:
                    # quick and partial off-line checksum validation
                    check_func = request.env["res.partner"].simple_vat_check
                vat_country, vat_number =\
                    request.env["res.partner"]._split_vat(data.get("vat"))
                if not check_func(vat_country, vat_number):  # simple_vat_check
                    error["vat"] = 'error'

            # acc_number validation
            if data.get('acc_number'):
                iban = data.get('acc_number')
                country_code_cli = False
                if data.get('acc_country_id'):
                    country = request.env['res.country'].sudo().search([
                        ('code', '=', data.get('acc_country_id'))])
                    country_code_cli = country.code.lower()
                iban = re.sub(r'[\W_]', '', iban or '')
                if not iban:
                    error["acc_number"] = 'error'
                    error_message.append(_("No IBAN !"))
                if country_code_cli in self._map_iban_template:
                    country_code = iban[:2].lower()
                    iban_template = self._map_iban_template.get(
                        country_code, False)
                    if (country_code_cli and
                            country_code_cli != country_code):
                        error["acc_number"] = 'error'
                        error_message.append(
                            _("Account Country and Bank Country "
                              "must be the same!"))
                    elif country_code not in self._map_iban_template:
                        error["acc_number"] = 'error'
                        error_message.append(
                            _("The IBAN is invalid, it should begin with "
                              "the country code"))
                    elif len(iban) != len(iban_template.replace(' ', '')):
                        error["acc_number"] = 'error'
                        error_message.append(
                            _("The IBAN does not seem to be correct. You "
                              "should have entered something like this "
                              "%s\n Where B = National bank code, "
                              "S = Branch code, C = Account No, "
                              "k = Check digit")
                            % iban_template)

                    check_chars = iban[4:] + iban[:4]
                    # BASE 36: 0..9,A..Z -> 0..35
                    digits = int(
                        ''.join(str(int(char, 36))
                                for char in check_chars))
                    if digits % 97 != 1:
                        error["acc_number"] = 'error'
                        error_message.append(
                            _("This IBAN does not pass the validation "
                              "check, please verify it."))

            # swift validation
            if not data.get('bic') and data.get(
                    'acc_number')[:2].lower() != 'es':
                if data.get('acc_country_id') != 'US':
                    error["bic"] = 'error'
                    error_message.append(
                        _('If the bank is not Spanish, you have to fill '
                          'the swift field.'))
            if data.get('bic'):
                if len(data.get('bic')) != 8 and len(
                        data.get('bic')) != 11:
                    error["bic"] = 'error'
                    error_message.append(
                        _('Invalid Swift! The length has to be 8 or 11.'))

            # US Bank validation
            if data.get('acc_country_id', False) == 'US':
                if (not data.get('aba') and
                        not data.get(
                            'fed_wire') and not data.get('bic')):
                    error["aba"] = 'error'
                    error["fed_wire"] = 'error'
                    error["bic"] = 'error'
                    error_message.append(
                        _('If the bank is form U.S., you have to add '
                          'at least the ABA, the FED wire or the swift.'))
            # Paypal email validation
            if (data.get('paypal_email') and
                    not tools.single_email_re.match(
                        data.get('paypal_email'))):
                error["email"] = 'error'
                error_message.append(
                    _('Invalid Email! Please enter a valid PayPal '
                      'email address.'))
        # error message for empty required fields
        if [err for err in error.values() if err == 'missing']:
            error_message.append(_('Some required fields are empty.'))

        unknown = ([k for k in data.iterkeys() if k not in
                    self.MANDATORY_DETAILS_FIELDS +
                    self.OPTIONAL_DETAILS_FIELDS])
        if unknown:
            error['common'] = 'Unknown field'
            error_message.append("Unknown field '%s'" % ','.join(unknown))

        return error, error_message

    """
    Function that generates the request of modification of the details
    of the user portal.
    """
    def confirm_details(self, partner, data):
        confirm = request.env['confirm.partner.website']
        bank = False

        if not request.env['res.country'].sudo().search([
                    ('id', '=', data.get('country_id'))]).state_ids:
            data['state_id'] = False

        acc_country_obj = False
        if data.get("acc_country_id", False) != "US":
            data['aba'] = ''
            data['fed_wire'] = ''
            acc_country = request.env['res.country'].sudo().search([
                    ('code', '=', data.get('acc_country_id'))])
            if acc_country:
                acc_country_obj = acc_country.id

        # Busqueda del banco
        if data.get("bic", False):
            bank_id = request.env['res.bank'].sudo().search([
                ('bic', '=', data.get("bic"))], limit=1)
            bank = bank_id.id

        if data.get("aba", False) and not bank:
            bank_id = request.env['res.bank'].sudo().search([
                ('aba', '=', data.get("aba"))], limit=1)
            bank = bank_id.id

        if data.get("fed_wire", False) and not bank:
            bank_id = request.env['res.bank'].sudo().search([
                ('fed_wire', '=', data.get("fed_wire"))], limit=1)
            bank = bank_id.id

        if data.get("bank_name", False) and not bank:
            bank_id = request.env['res.bank'].sudo().search([
                ('name', '=', data.get("bank_name"))], limit=1)
            bank = bank_id.id

        if partner.parent_id and partner.parent_id.bank_ids:
            benef = partner.parent_id.bank_ids[0].beneficiary
            acc = partner.parent_id.bank_ids[0].acc_number
            country = partner.parent_id.bank_ids[0].acc_country_id.id
            if partner.parent_id.bank_ids[0].bank_id:
                old_bank = partner.parent_id.bank_ids[0].bank_id.id
                bic = partner.parent_id.bank_ids[0].bank_id.bic
                aba = partner.parent_id.bank_ids[0].bank_id.aba
                fed_wire =\
                    partner.parent_id.bank_ids[0].bank_id.fed_wire
                bank_name = partner.parent_id.bank_ids[0].bank_id.name
            else:
                bic = False
                bank_name = False
                aba = False
                fed_wire = False
                old_bank = False

        elif partner.bank_ids:
            benef = partner.bank_ids[0].beneficiary
            acc = partner.bank_ids[0].acc_number
            country = partner.bank_ids[0].acc_country_id.id
            if partner.bank_ids[0].bank_id:
                old_bank = partner.bank_ids[0].bank_id.id
                bic = partner.bank_ids[0].bank_id.bic
                aba = partner.bank_ids[0].bank_id.aba
                fed_wire = partner.bank_ids[0].bank_id.fed_wire
                bank_name = partner.bank_ids[0].bank_id.name
            else:
                bic = False
                bank_name = False
                aba = False
                fed_wire = False
                old_bank = False

        else:
            benef = False
            acc = False
            country = False
            bic = False
            bank_name = False
            aba = False
            fed_wire = False
            old_bank = False

        if partner.parent_id:
            email_paypal = partner.parent_id.paypal_email
        else:
            email_paypal = partner.paypal_email

        dict_newf = {
            'new_name': data.get("name", False),
            'new_email': data.get("email", False),
            'new_phone': data.get("phone", False),
            'new_mobile': data.get("mobile", False),
            'new_company': data.get("company_name", False),
            'new_vat': data.get("vat", False),
            'new_street': data.get("street", False),
            'new_street2': data.get("street2", False),
            'new_city': data.get("city", False),
            'new_zip': data.get("zipcode", False),
            'new_state_id': data.get("state_id", False),
            'new_country_id': data.get("country_id", False),
            'new_bank': data.get("acc_number", False),
            'new_bank_country_id': acc_country_obj,
            'new_bank_name': data.get("bank_name", False),
            'new_swift': data.get("bic", False),
            'new_aba': data.get("aba", False),
            'new_fed_wire': data.get("fed_wire", False),
            'bank_id': bank,
            'new_beneficiary': data.get("beneficiary", False),
            'new_paypal_email': data.get("paypal_email", False),
            'new_comment_invoice': data.get('comment_invoice', False),
            }
        part_website = confirm.sudo().create({
            'partner_id': partner.id,
            'old_name': partner.name,
            'old_email': partner.email,
            'old_phone': partner.phone,
            'old_mobile': partner.mobile,
            'old_company': partner.company_name,
            'old_vat': partner.vat,
            'old_street': partner.street,
            'old_street2': partner.street2,
            'old_city': partner.city,
            'old_zip': partner.zip,
            'old_country_id': partner.country_id.id,
            'old_state_id': partner.state_id.id,
            'old_bank': acc,
            'old_bank_country_id': country,
            'old_bank_name': bank_name,
            'old_swift': bic,
            'old_aba': aba,
            'old_fed_wire': fed_wire,
            'old_beneficiary': benef,
            'old_paypal_email': email_paypal,
            'old_comment_invoice': partner.comment_invoice_portal,
            'new_name': data.get("name", False),
            'new_email': data.get("email", False),
            'new_phone': data.get("phone", False),
            'new_mobile': data.get("mobile", False),
            'new_company': data.get("company_name", False),
            'new_vat': data.get("vat", False),
            'new_street': data.get("street", False),
            'new_street2': data.get("street2", False),
            'new_city': data.get("city", False),
            'new_zip': data.get("zipcode", False),
            'new_state_id': data.get("state_id", False),
            'new_country_id': data.get("country_id", False),
            'new_bank': data.get("acc_number", False),
            'new_bank_country_id': acc_country_obj,
            'new_bank_name': data.get("bank_name", False),
            'new_swift': data.get("bic", False),
            'new_aba': data.get("aba", False),
            'new_fed_wire': data.get("fed_wire", False),
            'bank_id': bank,
            'old_bank_id': old_bank,
            'new_beneficiary': data.get("beneficiary", False),
            'new_paypal_email': data.get("paypal_email", False),
            'new_comment_invoice': data.get('comment_invoice', False),
            })

        part_website._test_checks(dict_newf)

    _map_iban_template = {
        'ad': 'ADkk BBBB SSSS CCCC CCCC CCCC',  # Andorra
        'ae': 'AEkk BBBC CCCC CCCC CCCC CCC',  # United Arab Emirates
        'al': 'ALkk BBBS SSSK CCCC CCCC CCCC CCCC',  # Albania
        'at': 'ATkk BBBB BCCC CCCC CCCC',  # Austria
        'az': 'AZkk BBBB CCCC CCCC CCCC CCCC CCCC',  # Azerbaijan
        'ba': 'BAkk BBBS SSCC CCCC CCKK',  # Bosnia and Herzegovina
        'be': 'BEkk BBBC CCCC CCXX',  # Belgium
        'bg': 'BGkk BBBB SSSS DDCC CCCC CC',  # Bulgaria
        'bh': 'BHkk BBBB CCCC CCCC CCCC CC',  # Bahrain
        'br': 'BRkk BBBB BBBB SSSS SCCC CCCC CCCT N',  # Brazil
        'ch': 'CHkk BBBB BCCC CCCC CCCC C',  # Switzerland
        'cr': 'CRkk BBBC CCCC CCCC CCCC C',  # Costa Rica
        'cy': 'CYkk BBBS SSSS CCCC CCCC CCCC CCCC',  # Cyprus
        'cz': 'CZkk BBBB SSSS SSCC CCCC CCCC',  # Czech Republic
        'de': 'DEkk BBBB BBBB CCCC CCCC CC',  # Germany
        'dk': 'DKkk BBBB CCCC CCCC CC',  # Denmark
        'do': 'DOkk BBBB CCCC CCCC CCCC CCCC CCCC',  # Dominican Republic
        'ee': 'EEkk BBSS CCCC CCCC CCCK',  # Estonia
        'es': 'ESkk BBBB SSSS KKCC CCCC CCCC',  # Spain
        'fi': 'FIkk BBBB BBCC CCCC CK',  # Finland
        'fo': 'FOkk CCCC CCCC CCCC CC',  # Faroe Islands
        'fr': 'FRkk BBBB BGGG GGCC CCCC CCCC CKK',  # France
        'gb': 'GBkk BBBB SSSS SSCC CCCC CC',  # United Kingdom
        'ge': 'GEkk BBCC CCCC CCCC CCCC CC',  # Georgia
        'gi': 'GIkk BBBB CCCC CCCC CCCC CCC',  # Gibraltar
        'gl': 'GLkk BBBB CCCC CCCC CC',  # Greenland
        'gr': 'GRkk BBBS SSSC CCCC CCCC CCCC CCC',  # Greece
        'gt': 'GTkk BBBB MMTT CCCC CCCC CCCC CCCC',  # Guatemala
        'hr': 'HRkk BBBB BBBC CCCC CCCC C',  # Croatia
        'hu': 'HUkk BBBS SSSC CCCC CCCC CCCC CCCC',  # Hungary
        'ie': 'IEkk BBBB SSSS SSCC CCCC CC',  # Ireland
        'il': 'ILkk BBBS SSCC CCCC CCCC CCC',  # Israel
        'is': 'ISkk BBBB SSCC CCCC XXXX XXXX XX',  # Iceland
        'it': 'ITkk KBBB BBSS SSSC CCCC CCCC CCC',  # Italy
        'jo': 'JOkk BBBB NNNN CCCC CCCC CCCC CCCC CC',  # Jordan
        'kw': 'KWkk BBBB CCCC CCCC CCCC CCCC CCCC CC',  # Kuwait
        'kz': 'KZkk BBBC CCCC CCCC CCCC',  # Kazakhstan
        'lb': 'LBkk BBBB CCCC CCCC CCCC CCCC CCCC',  # Lebanon
        'li': 'LIkk BBBB BCCC CCCC CCCC C',  # Liechtenstein
        'lt': 'LTkk BBBB BCCC CCCC CCCC',  # Lithuania
        'lu': 'LUkk BBBC CCCC CCCC CCCC',  # Luxembourg
        'lv': 'LVkk BBBB CCCC CCCC CCCC C',  # Latvia
        'mc': 'MCkk BBBB BGGG GGCC CCCC CCCC CKK',  # Monaco
        'md': 'MDkk BBCC CCCC CCCC CCCC CCCC',  # Moldova
        'me': 'MEkk BBBC CCCC CCCC CCCC KK',  # Montenegro
        'mk': 'MKkk BBBC CCCC CCCC CKK',  # Macedonia
        'mr': 'MRkk BBBB BSSS SSCC CCCC CCCC CKK',  # Mauritania
        'mt': 'MTkk BBBB SSSS SCCC CCCC CCCC CCCC CCC',  # Malta
        'mu': 'MUkk BBBB BBSS CCCC CCCC CCCC CCCC CC',  # Mauritius
        'nl': 'NLkk BBBB CCCC CCCC CC',  # Netherlands
        'no': 'NOkk BBBB CCCC CCK',  # Norway
        'pk': 'PKkk BBBB CCCC CCCC CCCC CCCC',  # Pakistan
        'pl': 'PLkk BBBS SSSK CCCC CCCC CCCC CCCC',  # Poland
        'ps': 'PSkk BBBB XXXX XXXX XCCC CCCC CCCC C',  # Palestinian
        'pt': 'PTkk BBBB SSSS CCCC CCCC CCCK K',  # Portugal
        'qa': 'QAkk BBBB CCCC CCCC CCCC CCCC CCCC C',  # Qatar
        'ro': 'ROkk BBBB CCCC CCCC CCCC CCCC',  # Romania
        'rs': 'RSkk BBBC CCCC CCCC CCCC KK',  # Serbia
        'sa': 'SAkk BBCC CCCC CCCC CCCC CCCC',  # Saudi Arabia
        'se': 'SEkk BBBB CCCC CCCC CCCC CCCC',  # Sweden
        'si': 'SIkk BBSS SCCC CCCC CKK',  # Slovenia
        'sk': 'SKkk BBBB SSSS SSCC CCCC CCCC',  # Slovakia
        'sm': 'SMkk KBBB BBSS SSSC CCCC CCCC CCC',  # San Marino
        'tn': 'TNkk BBSS SCCC CCCC CCCC CCCC',  # Tunisia
        'tr': 'TRkk BBBB BRCC CCCC CCCC CCCC CC',  # Turkey
        'vg': 'VGkk BBBB CCCC CCCC CCCC CCCC',  # Virgin Islands
        'xk': 'XKkk BBBB CCCC CCCC CCCC',  # Kosovo
    }
